// mozileTB.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Toolbar Javascript
mozile version 0.52

Created by James A. Overton
Created on 2003-07-30

Last revised by James A. Overton
Last revised on 2003-11-19

DESCRIPTION:
This file handles the toolbar for Mozile when it is installed as chrome.
It is called when the user clicks in the window (mozileMouse), presses
a key (mozileKey) or clicks a button/menuitem (mozileFormat and mozileCommand).
Internal to the toolbar are two groups of functions. The first handles
the do, undo, and redo stuff, and the second handles the clipboard stuff.
I've done my best to document each function where it is written.

*********/

/********* Global Variables *********/

var mozileTBVersion = "0.52";

var mozileCaretDefault = false;	// remember the state of the caret that the user has set, so it can be restored

var mozileTarget = "";				// the currently focussed window, if it has loaded Mozile.
var mozileSelection = "";			// the eDOM selection of the currently focussed window.
var mozilePrefs = "";					// Netscape preferences component

var keyMax = 20;							// the number of key presses before the state is saved to the Undo Stack.
var undoMax = 100;						// the number of undo steps available.



/********* Handlers *********/

// Listen for clicks, and then decide whether or not the Mozile toolbar needs to be shown or hidden
document.addEventListener("click", mozileMouse, false);

// Losten for keypresses, and decide what to do
document.addEventListener("keypress", mozileKeys, true);

// Run this function on load.
mozileTBStart();



/********* Primary Functions *********/


// Mozile TB Start
// Initializes the mozilePrefs and mozileCaretActive variables.

function mozileTBStart() {

			// the caret requires added privileges
	netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
	mozilePrefs = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch(null);
	mozileCaretDefault = mozilePrefs.getBoolPref('accessibility.browsewithcaret');		// remember the previous state of the caret so it can be restored

} // end mozileTBStart()




// Mozile Detect
// Check for the mozileChromeLoaded flag.
// Check to see if the user has selected an editable area.
// If so, set the global mozileTarget and mozileSelection variables.
// If not, return 0, and don't do anything else.

function mozileDetect() {

		// if the currently focussed window does not have the mozileChromeLoaded flag set, then Mozile shouldn't do anything.
	if(!document.commandDispatcher.focusedWindow.mozileChromeLoaded
			|| document.commandDispatcher.focusedWindow.mozileChromeLoaded != "true") {
		return 0;
	}

	mozileTarget = document.commandDispatcher.focusedWindow;

		// undo/redo can change the selection to the documentElement, so ignore a sudden change
	if(mozileTarget.document.documentElement == mozileSelection.anchorNode) {
		return 1;
	}

	mozileSelection = mozileTarget.getSelection();

		// test the parent node of the node containing the selection to see if it is editable.
	if(!mozileSelection.anchorNode.parentNode.userModifiable) {
		return 0;
	}

	return 1;

} // end mozileDetect()




// Mozile Mouse
// Handles mouse click events
// Decides whether to autohide the toolbar, and when to activate the caret.

function mozileMouse(event) {

	//alert("Tag "+event.target.tagName);
		// this fixes an annoying bug, where clicking on a tab could unhide the toolbar.
	if(event.target.tagName=="tabbrowser") {
		return;
	}
		// if one of the tags on this list has been clicked on, then focus it. Fixes another bug.
	var focusList = new Array("input","textarea");
	for(var i=0; i<focusList.length; i++) {
		if(event.target.tagName.toLowerCase()==focusList[i]) {
			event.target.focus();
			return;
		}
	}

		// set the toolbar and autohide variables
	var toolbar = document.getElementById("mozile-toolbar");
	var autohide = document.getElementById('mozile-autohide');		// the XUL menuitem which enables/disables autohiding

		// if Mozile is detected, then show the toolbar if it is hidden, and turn the caret on
	if(mozileDetect()) {
		if(toolbar.collapsed) {
			toolbar.collapsed = !toolbar.collapsed;		// show toolbar
		}
		mozilePrefs.setBoolPref('accessibility.browsewithcaret', true);
	}
		// if Mozile is not detected, then hide the toolbar (if autohide is on) and restore the caret state.
	else {
		if(!toolbar.collapsed && autohide.getAttribute('checked') == "true")	{
			toolbar.collapsed = !toolbar.collapsed;		// hide toolbar
		}
		mozilePrefs.setBoolPref('accessibility.browsewithcaret', mozileCaretDefault);		// restore the previous caret state
	}


} // end mozileMouse




// Mozile Keys
// Handles keyboard events, counts keys so mozileDo("Typing") can be called
// Most of the work is done in mozileTBKeys.js

function mozileKeys(event) {

		// if Mozile is not detected, do nothing.
	if(!mozileDetect()) {
		return;
	}
	var handled = false;
	var ignoreKeys = new Array(37,38,39,40);		// ignore the arrow keys


		// metakey is Mac OSX standard
	if(event.ctrlKey || event.metaKey) {
			// Handle control characters
		handled = ctrlKeyPressHandler(event);
	}

		// handle non-control characters
	else {
			// we want to call mozileDo after [keyMax] characters are entered, so typing can be undone.
			// first makes sure this isn't an arrow key
		var ignore=false;
		for(var i=0; i<ignoreKeys.length; i++) {
			if(event.keyCode==ignoreKeys[i]) {
				ignore=true;
				break;
			}
		}

			// then call mozileDo if the keyCount exceeds the keyMax, otherwise increment the keyCount
		if(!ignore) {
				// create the keycount if needed, and save the current state.
			if(!mozileTarget.keyCount || mozileTarget.keyCount>=keyMax) {
				mozileDo("Typing");
				mozileTarget.keyCount=1;
			}
			else {
				mozileTarget.keyCount++;
			}
		}

			// now handle the keystroke
		handled = nonctrlKeyPressHandler(event);
	}


	// If the event has been handled don't let things go any further.
	if(handled)
	{
		//cancel event: TODO05: why all three?
		event.stopPropagation();
		event.returnValue = false;
  		event.preventDefault();
  		return false;
	}

} // end mozileKeys()






/********* Button/MenuItem Handling Functions *********/

// Mozile Format
// This is called by formatting commands of all sorts.
// It looks the command key up in a big array, which is stored in mozileTBCOmmands.js
// Then it follows the instructions in the array, collaing various eDOM functions

function mozileFormat(key) {
		// make sure Mozile is detected
	if(!mozileDetect()) {
		alert("No selection for Mozile to edit!");
		return;
	}

		// if there is no entry in the array, throw an error
	if(!mozileTBCommands[key]) {
		alert('Command  "'+ key +'" not defined!');
	}
	var command = mozileTBCommands[key];
	var name = command[0];
	var tip = command[1];			// a tooltip, not yet used
	var type = command[2];

		// the main switch
	switch(type) {
		case "toggleStyle":
			mozileDo(name);
			mozileSelection.toggleTextStyle(command[3], command[4], command[5]);
			break;

		case "linesContainer":
			mozileDo(name);
			mozileSelection.changeLinesContainer(command[3]);
			break;

		case "styleText":
			mozileDo(name);
			mozileSelection.styleText(command[3], command[4]);
			break;

		case "styleLines":
			mozileDo(name);
			mozileSelection.styleLines(command[3], command[4]);
			break;

		case "toggleListLines":
			mozileDo(name);
			mozileSelection.toggleListLines(command[3], command[4]);
			break;

				// some commands don't fit into the other groups, run another switch
		case "special":
			switch(command[3]) {
				case "indent":
					mozileDo(name);
					mozileSelection.indentLines();
					break;

				case "outdent":
					mozileDo(name);
					mozileSelection.outdentLines();
					break;

				case 'link':
					if(mozileSelection.isCollapsed) // must have a selection or don't prompt
						return;
							var href = prompt("Enter a URL:", "");
					if(href == null) // null href means prompt canceled - BUG FIX FROM Karl Guertin
						return;
					if(href != "") {
						mozileDo(name);
						mozileSelection.linkText(href);
					}
					else {
						mozileDo(name);
						mozileSelection.clearTextLinks();
					}
					break;

				case "unlink":
					mozileDo(name);
					mozileSelection.clearTextLinks();
					break;

				case "unformat":
					mozileDo(name);
					mozileSelection.removeLinesContainer();
					break;

				case 'table':
					alert("Table insertion coming soon ...");
					break;

				case 'image': // need to replace with native file selection dialog
							var imgref = prompt("Enter the image url or file name:", "");
					if(imgref == null) // null href means prompt canceled
						return;
					if(imgref == "")
						return; // ok with no name filled in
					var img = mozileTarget.documentCreateXHTMLElement("img");
					img.src = imgref; // any way to tell if it is valid?
					mozileDo(name);
					mozileSelection.insertNode(img);
					break;

				case 'hr':
					var hr = mozileTarget.documentCreateXHTMLElement("hr");
					mozileDo(name);
					mozileT.insertNode(hr);
					break;
			}
		break;

			// if the command type hasn't been found, throw an error
		default:
			alert('Command type "'+ command[2] +'" not defined!');
		break;
	}

} // end mozileFormat()


// Mozile Command
// This function is used for non-formatting commands
// All of the behaviours are in here, or called from here.

function mozileCommand(name) {
		// make sure Mozile is detected
	if(!mozileDetect()) {
		alert("No editable selection!");
		return;
	}

		// run over the switch, some of these are iffy, so they're in try blocks
	switch(name) {
		case 'undo':
			try {
				mozileUndo();
			} catch(e) { alert(e); }
			break;
		case 'redo':
			try {
				mozileRedo();
			} catch(e) { alert(e); }
			break;
		case 'copy':
			try {
				mozileCopy();
			} catch(e) { alert(e); }
			break;
		case 'cut':
			try {
				mozileCut();
			} catch(e) { alert(e); }
			break;
		case 'paste':
			try {
				mozilePaste();
			} catch(e) { alert(e); }
			break;

			// Saving is handled by mozileTBSave.js
		case "save":
			try {
				mozileSave();
			} catch(e) { alert("mozileCommand(save): " + e); }
			break;

		// Dialogs and Other Stuff
		case "update":
			alert("TODO: Check for an update!");
			break;
		case "settings":
			window.open('chrome://mozile/content/settings/settings.xul', "Settings", "chrome");
			break;
		case "home":
			window.open('http://mozile.mozdev.org');
			break;
		case "about":
			var msg =
				"Mozile, the Mozilla Inline Editor \n Version Code: " +
				mozileTBVersion + "\n " +
				"Licensed Under: MPL 1.1/GPL 2.0/LGPL 2.1 \n Full Terms at http://mozile.mozdev.org/license.html";
			alert(msg);
			break;
		case "info":
			var msg =
				"Page Information (Mozile Specific)\n" +
				"Location: \n" +
				mozileTarget.document.location + "\n" +
				"Configuration File: \n" +
				mozileTarget.MOZILE_CONFIG_DOC_DIR + "mozileConfig.xml\n" +
				"";
			alert(msg);
			break;
		case "help":
			window.open('http://mozile.mozdev.org/help.html');
			break;
	}
}	// end mozileCommand()





/********** Undo Functionality *************/

// This is the lazy way, I copy the whole BODY...
// This code still needs to be tweaked.


// Mozile Do
// Records the state of the current document into the undoStack for that document,
// and increments the undoCounter (each document gets its own stack and counter).
// All steps higher up in the stack are eliminated when a lower on e is added.
// If the undoMax number of steps is hit, then the stack is spliced.
// The undoStack includes the name of the command and the whole BODY node.

// Yes, it is a brute force solution to copy the whole BODY,
// but it is the best solution for multiple contentEditable areas,
// (doing the whole HTML messes up the scrollbar)
// and the performance seems fine.
// Of course this won't work for XML in general...

function mozileDo(name) {
	try{
		if(!mozileTarget.undoStack) {
			mozileTarget.undoStack=new Array();
			mozileTarget.undoCounter=0;
		}
		var undoStep = new Array;
		var getNode=mozileTarget.document.documentElement.childNodes[2];
		undoStep['name']=name;
		undoStep['node']=getNode.cloneNode(true);
		//undoStep['position']=getPosition(getNode);
		mozileTarget.undoStack=mozileTarget.undoStack.slice(0,mozileTarget.undoCounter);			// make sure there are no extra steps
		if(mozileTarget.undoStack.length > undoMax) {
			mozileTarget.undoStack=mozileTarget.undoStack.slice(1);			// make sure there are no extra steps
		}
		mozileTarget.undoStack.push(undoStep);
		mozileTarget.undoCounter=mozileTarget.undoStack.length;
		//alert("Undo:"+mozileTarget.undoStack.length +":"+mozileTarget.undoCounter);
		//updateUndoButtons();
	} catch(e) { alert("mozileDo(): "+e); }
} // end mozileDo



// Update Undo Buttons
// This works but is not currently used.
// It gives the user some feedback about when more undo/redo steps are available.

function updateUndoButtons() {
	var undo=document.getElementById('mozile-undo');
	var redo=document.getElementById('mozile-redo');
	if(mozileTarget.undoCounter<1) undo.setAttribute('disabled','true');
	else undo.removeAttribute('disabled');
	if(mozileTarget.undoCounter>mozileTarget.undoStack.length-2) redo.setAttribute('disabled','true');
	else redo.removeAttribute('disabled');
} // end updateUndoButtons()

// Dump Undo Stack
// Mostly for debugging

function dumpUndoStack() {
	var out="Current: "+mozileTarget.undoCounter+"\n";
	for(var i=0;i<mozileTarget.undoStack.length;i++) {
		cur=mozileTarget.undoStack[i];
		out+="["+i+"] "+cur['name'] +": "+ cur['node'].tagName +"\n";
	}
	alert(out);
} // end dumpUndoStack()




// Mozile Undo
// Moves doen the undoStack from the curent undoCounter,
// replacing the current BODY node with the one from the stack.
// Behaves a bit strangely around the undoMax limit.

function mozileUndo() {
	if(!mozileTarget.undoStack) {
		mozileTarget.undoStack=new Array();
		mozileTarget.undoCounter=0;
		return;
	}
	var undoStack=mozileTarget.undoStack;				// it is safe to copy this, since it will not be manipulated

	if(mozileTarget.undoCounter<1) {
		//alert("Nothing to Undo!");
		return;
	}
	if(mozileTarget.undoCounter==undoStack.length) {
		//alert("Current");
		mozileDo('Current');
 		mozileTarget.undoCounter--;
	}
 	mozileTarget.undoCounter--;
	var undoStep=undoStack[mozileTarget.undoCounter];

	var replacement=undoStep['node'];
	var undoTarget=mozileTarget.document.documentElement.childNodes[2];
	var parent=undoTarget.parentNode;
	parent.replaceChild(replacement,undoTarget);
	//updateUndoButtons();
} // end mozileUndo()



// Mozile Redo
// Moves up the undoStack from the current undoCounter,
// replacing the current BODY noe with the one in the stack.

function mozileRedo() {
	if(!mozileTarget.undoStack) {
		mozileTarget.undoStack=new Array();
		mozileTarget.undoCounter=0;
		return;
	}
	var undoStack=mozileTarget.undoStack;				// it is safe to copy this, since it will not be manipulated

	if(mozileTarget.undoCounter>undoStack.length-2) {
		//alert("Nothing to Redo!");
		return;
	}
 	mozileTarget.undoCounter++;
	var undoStep=undoStack[mozileTarget.undoCounter];

	var replacement=undoStep['node'];
	var undoTarget=mozileTarget.document.documentElement.childNodes[2];
	var parent=undoTarget.parentNode;
	parent.replaceChild(replacement,undoTarget);
	//updateUndoButtons();
} // end mozileRedo()










/********** Clipboard Functionality *************/


// Mozile Paste
// Takes the content of the system clipboard and inserts it at the current selection.
// Right now only plain text is handled.

function mozilePaste() {
	mozileDo("Paste");

	var clipboard = mozilla.getClipboard();
	// only get text for now: POST04: need XHTML validator first so that correct
	// HTML comes first.
	var text = clipboard.getData(MozClipboard.TEXT_FLAVOR);
	if(text)
	{
		mozileSelection.insertNode(mozileTarget.document.createTextNode(text));
	}
} // end mozilePaste()



// Mozile Copy
// Copies the selected text to the system clipboard.
// The selection is currently copied as plain text.

function mozileCopy() {
	mozileDo("Copy");

	var cssr = mozileSelection.getEditableRange();

	if(!cssr || cssr.collapsed) // not an editable area or nothing selected
		return;

	// data to save - render as text (temporary thing - move to html later)
	var text = cssr.toString();

	var clipboard = mozilla.getClipboard();

	// clipboard.setData(deletedFragment.saveXML(), "text/html"); // go back to this once, paste supports html paste!
	clipboard.setData(text, MozClipboard.TEXT_FLAVOR);
} // end mozileCopy()




// Mozile Cut
// Copy the current selection to the system clipboard, then remove it from the document.
// Works in plain text.

function mozileCut() {
	mozileDo("Cut");

	var cssr = mozileSelection.getEditableRange();

	if(!cssr || cssr.collapsed) // not an editable area or nothing selected
		return;

	var clipboard = mozilla.getClipboard();

	// data to save - render as text (temporary thing)
	var text = cssr.toString();

	var deletedFragment = cssr.extractContentsByCSS();

	// clipboard.setData(deletedFragment.saveXML(), MozClipboard.HTML_FLAVOR); // go back to this once, paste supports html paste!
	clipboard.setData(text, MozClipboard.TEXT_FLAVOR);

	mozileSelection.removeAllRanges();
	mozileSelection.addRange(cssr);
} // end mozileCut()




/********** Miscellaneous Functionality *************/

// Mozile Toggle Caret
// Toggles the caret and the mozileCaretDefault variable on and off.

function mozileToggleCaret() {
	mozileCaretDefault = !mozileCaretDefault;
	mozilePrefs.setBoolPref('accessibility.browsewithcaret', mozileCaretDefault);		// restore
} // end mozileToggleCaret()
