// install.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		James A. Overton <james@overton.ca>
 *
 * ***** END LICENSE BLOCK ***** */

/*
 *
 * Mozile Installation Javascript
 * mozile version 0.52
 *
 * Created by James A. Overton
 * Created on 2003-07-30
 *
 * Last revised by James A. Overton
 * Last revised on 2003-11-19
 *
 *
 * ACKNOWLEDGEMENTS:
 * I have adapted this file from Chris Pederick's Web Devloper 0.3 extension for Firebird, and I am very grateful to him.
 * See http://chrispederick.myacen.com/work/firebird/webdeveloper/
 *
 *
 * DESCRIPTION:
 * This file is the part of mozile.xpi that does the real work.
 * It first checks for an installed version, both in the main and profile chrome directories,
 * and gives the user the option to install over top of the old copy.
 * Then it gives the option to install in the profile or main chrome directories,
 * but only where Mozile hasn't already been installed.
 * The browser must be restarted before the overlays kick in.
 *
 *
 * TODO:
 * - Better error messgaes, expecially when trying to install over another version.
 * - Uninstall!
 *
 */


var author              = "mozile.mozdev.org";
var displayName         = "Mozile Editor";
var name                = "mozile";
var version             = "0.52";

var contentFlag         = CONTENT | PROFILE_CHROME;
var localeFlag          = LOCALE | PROFILE_CHROME;
var skinFlag            = SKIN | PROFILE_CHROME;
var folder              = getFolder("Profile", "chrome");
var contentFolder				= "content/";
var localeFolder				= "locale/en-US/";
var skinFolder					= "skin/classic/";

var message1						= "Mozile is already installed in the application folder. Would you like to replace the installed version with this version?";
var message2						= "Mozile is already installed in your profile folder. Would you like to replace the installed version with this version?";
var message3						= "Do you want to install this extension to your profile folder? This way the extension won't be lost when you update your browser, but it will not be accessible from other profiles.";
var message4						= "Do you want to install this extension to the application folder?";

var error               = null;
var jarName             = name + ".jar";
var existsInApplication = File.exists(getFolder(getFolder("chrome"), jarName));
var existsInProfile     = File.exists(getFolder(folder, jarName));

var doneFlag=false;

function finish(error) {
	// If the install failed
	if(error != SUCCESS && error != 999)
	{
		alert("The installation of the extension failed.\n" + error);
		cancelInstall(error);
	}
	else {
		alert("The extension installed successfully! Please restart your browser.");
	}
	doneFlag=true;
	cancelInstall(error);
}

initInstall(displayName, name, version);

// check for application install, if yes ask if the user wants to update
if(existsInApplication && !doneFlag) {
	yn=confirm(message1);
	if(yn) {
    contentFlag = CONTENT | DELAYED_CHROME;
		localeFlag  = LOCALE | DELAYED_CHROME;
    skinFlag    = SKIN | DELAYED_CHROME;
    folder      = getFolder("chrome");

		setPackageFolder(folder);
		error = addFile(author, version, jarName, folder, null);

    error = performInstall();
		finish(error);
	}
}

// check for a profile install, if yes ask if the user wants to update
if(existsInProfile && !doneFlag) {
	yn=confirm(message2);
	if(yn) {
		setPackageFolder(folder);
		error = addFile(author, version, jarName, folder, null);

    error = performInstall();
		finish(error);
	}
}

// ask if the user wants to install to profile
if(!doneFlag && !existsInProfile && confirm(message3)) {
		setPackageFolder(folder);
		error = addFile(author, version, jarName, folder, null);    folder = getFolder(folder, jarName);

    registerChrome(contentFlag, folder, contentFolder);
    registerChrome(localeFlag, folder, localeFolder);
    registerChrome(skinFlag, folder, skinFolder);

    error = performInstall();

		finish(error);
}

// ask if the user wants to install to application
if(!doneFlag && !existsInApplication && confirm(message4)) {
    contentFlag = CONTENT | DELAYED_CHROME;
		localeFlag  = LOCALE | DELAYED_CHROME;
    skinFlag    = SKIN | DELAYED_CHROME;
    folder      = getFolder("chrome");

		setPackageFolder(folder);
		error = addFile(author, version, jarName, folder, null);    folder = getFolder(folder, jarName);

    registerChrome(contentFlag, folder, contentFolder);
    registerChrome(localeFlag, folder, localeFolder);
    registerChrome(skinFlag, folder, skinFolder);

    error = performInstall();

		finish(error);
}


// Otherwise
if(!doneFlag) {
	alert("The installation of the extension failed.\n" + error);
	cancelInstall(error);
}

